@@ -1,6 +1,7 @@
 
-denstrip <- function(x, dens, at, width, horiz=TRUE, colmax=par("fg"), scale=1, gamma=1, 
-                     ticks=NULL, tlen=1.5, twd=par("lwd"), mticks=NULL, mlen=1.5, mwd=par("lwd")*2, 
+denstrip <- function(x, dens, at, width, horiz=TRUE, colmax, scale=1, gamma=1, 
+                     ticks=NULL, tlen=1.5, twd, mticks=NULL, mlen=1.5, mwd,
+                     lattice=FALSE,
                      ...)
 {
     if (!is.numeric(x)) stop("\'x\' must be numeric")
@@ -15,6 +16,21 @@ denstrip <- function(x, dens, at, width, horiz=TRUE, colmax=par("fg"), scale=1,
         dens <- dens[order(x)]
         x <- sort(x)
     }
+    if (lattice) {
+        rect.fn <- panel.rect; seg.fn <- panel.segments
+        default.width <- diff(trellis.last.object()[[if(horiz) "y.limits" else "x.limits"]]) / 30
+        default.colmax <- trellis.par.get("add.line")$col
+        default.twd <- trellis.par.get("add.line")$lwd; default.mwd <- trellis.par.get("add.line")$lwd*2
+    }
+    else {
+        rect.fn <- rect; seg.fn <- segments;
+        default.width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 30
+        default.colmax <- par("fg"); default.twd <- par("lwd"); default.mwd <- par("lwd")*2
+    }
+    if (missing(width)) width <- default.width
+    if (missing(colmax)) colmax <- default.colmax
+    if (missing(twd)) twd <- default.twd
+    if (missing(mwd)) mwd <- default.mwd
     dens <- dens / max(dens) * scale
     n <- length(x)
     rgbmax <- col2rgb(colmax, alpha=TRUE)
@@ -25,8 +41,6 @@ denstrip <- function(x, dens, at, width, horiz=TRUE, colmax=par("fg"), scale=1,
                 p*rgbmax[3] + (1 - p)*255, alpha=rgbmax[4], maxColorValue=255)
     first.col <- c(TRUE, cols[2:(n-1)] != cols[1:(n-2)])
     next.col <- c(first.col,TRUE); next.col[1] <- FALSE    
-    if (missing(width))
-        width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 30
     if (horiz) {
         xleft <- x[-n][first.col]; xright=x[next.col];
         ybottom <- at-width/2; ytop <- at+width/2
@@ -35,59 +49,79 @@ denstrip <- function(x, dens, at, width, horiz=TRUE, colmax=par("fg"), scale=1,
         xleft <- at-width/2; xright <- at+width/2;
         ybottom <- x[-n][first.col]; ytop <- x[next.col]
     }
-    rect(xleft=xleft, ybottom=ybottom, xright=xright, ytop=ytop,
+    rect.fn(xleft=xleft, ybottom=ybottom, xright=xright, ytop=ytop,
          border=NA, col = cols[first.col])
     if (!is.null(ticks)){
         if (horiz) { tx0 <- tx1 <- ticks; ty0 <- at-width*tlen/2; ty1 <- at+width*tlen/2 }
         else { tx0 <- at-width*tlen/2; tx1 <- at+width*tlen/2; ty0 <- ty1 <- ticks }
-        segments(tx0, ty0, tx1, ty1, lwd=twd)
+        seg.fn(tx0, ty0, tx1, ty1, lwd=twd)
     }
     if (!is.null(mticks)){
         if (horiz) { tmx0 <- tmx1 <- mticks; tmy0 <- at-width*mlen/2; tmy1 <- at+width*mlen/2  }
         else { tmx0 <- at-width*mlen/2; tmx1 <- at+width*mlen/2; tmy0 <- tmy1 <- mticks }
-        segments(tmx0, tmy0, tmx1, tmy1, lwd=mwd)
+        seg.fn(tmx0, tmy0, tmx1, tmy1, lwd=mwd)
     }
     invisible()
 }
 
 denstrip.normal <- function(mean, sd, log=FALSE, nx=1000, ...){    
     x <-
-        if (log) qlnorm(seq(0,1,length=nx)[-c(1,nx)], mean, sd)
-        else qnorm(seq(0,1,length=nx)[-c(1,nx)], mean, sd)
+        if (log) qlnorm(ppoints(nx, a=0), mean, sd)
+        else qnorm(ppoints(nx, a=0), mean, sd)
     dens <- if (log) dlnorm(x, mean, sd) else dnorm(x, mean, sd)
     denstrip(x=x, dens=dens, ...)
 }
 
 denstrip.legend <- function(x, # central x position 
-                            y, # central y position 
-                            width, len, colmax=par("fg"), gamma=1, horiz=FALSE,
-                            max, nticks=5, value.adj=0, main="Density")
+                            y, # central y position
+                            width, len, colmax, gamma=1, horiz=FALSE,
+                            max, nticks=5, ticks, value.adj=0, cex, main="Density", lattice=FALSE)
 {
-    if (missing(width))
-        width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 30
-    if (missing(len))
-        len <- diff(par("usr")[if(horiz) 1:2 else 3:4]) / 4
+    if (lattice) {
+        poly.fn <- panel.polygon; seg.fn <- panel.segments; text.fn <- panel.text
+        default.width <- diff(trellis.last.object()[[if(horiz) "y.limits" else "x.limits"]]) / 30
+        default.len <- diff(trellis.last.object()[[if(horiz) "x.limits" else "y.limits"]]) / 4
+        default.colmax <- trellis.par.get("add.line")$col
+        default.cex <- trellis.par.get("axis.text")$cex * 0.75
+      }
+    else {
+        poly.fn <- polygon; seg.fn <- segments; text.fn <- text
+        default.width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 30
+        default.len <- diff(par("usr")[if(horiz) 1:2 else 3:4]) / 4
+        default.colmax <- par("fg")
+        default.cex <- par("cex") * 0.75
+    }
+    if (missing(width)) width <- default.width
+    if (missing(len)) len <- default.len
+    if (missing(colmax)) colmax <- default.colmax
+    if (missing(cex)) cex <- default.cex
     if (horiz) {        
         pt <- x; at <- y
         xdim <- len; ydim <- width
     }
     else {
-        pt <- y; at <- x
-        xdim <- width; ydim <- len
+      pt <- y; at <- x
+      xdim <- width; ydim <- len
     }
     npoints <- 1000 # number of distinct colors. no need to make this an argument
     dx <- seq(pt - len/2, pt + len/2, length=npoints)
     ddens <- seq(0, max, length=npoints)
-    denstrip(x=dx, dens=ddens, at=at, width=width, colmax=colmax, gamma=gamma, horiz=horiz)    
-    polygon(x = c(x - xdim/2, x + xdim/2, x + xdim/2, x - xdim/2), # draw box around strip
+    denstrip(x=dx, dens=ddens, at=at, width=width, colmax=colmax, gamma=gamma, horiz=horiz, lattice=lattice)    
+    poly.fn(x = c(x - xdim/2, x + xdim/2, x + xdim/2, x - xdim/2), # draw box around strip
             y = c(y - ydim/2, y - ydim/2, y + ydim/2, y + ydim/2))
-    ticks.at <- seq(pt - len/2, pt + len/2, length=nticks)
-    tick.nos <- round(seq(0, max, length=nticks), 2)
-    if (horiz) segments(ticks.at, at - width/2, ticks.at, at - width*0.75)
-    else segments(at + width/2, ticks.at, at + width*0.75, ticks.at)
-    if (horiz) text(ticks.at, at - width - value.adj, tick.nos, cex = par()$cex * 0.75, pos=1)
-    else text(at + width + value.adj, ticks.at, tick.nos, cex = par()$cex * 0.75, pos=4)
-    text(x, y + ydim/2, main, cex = par()$cex * 0.75, pos=3)
+    if (missing(ticks)) {
+        ticks.at <- seq(pt - len/2, pt + len/2, length=nticks)
+        tick.nos <- signif(seq(0, max, length=nticks), 3)
+    }
+    else {
+        ticks.at <- pt - len/2 + ticks / max * len
+        tick.nos <- ticks
+    }
+    if (horiz) seg.fn(ticks.at, at - width/2, ticks.at, at - width*0.75)
+    else seg.fn(at + width/2, ticks.at, at + width*0.75, ticks.at)
+    if (horiz) text.fn(ticks.at, at - width - value.adj, tick.nos, cex = cex, pos=1)
+    else text.fn(at + width + value.adj, ticks.at, tick.nos, cex = cex, pos=4)
+    text.fn(x, y + ydim/2, main, cex = cex, pos=3)
     invisible()
 }
 
@@ -140,7 +174,7 @@ densregion.survfit <- function(x, ny=20, ...) {
     y <- matrix(nrow=n, ncol=ny) 
     ## "ny" ordinates, based on normal quantiles, where density must be calculated for each event time
     for (i in 1:n)
-        y[i,] <- invtr(qnorm(seq(0,1,length=ny+2)[-c(1,ny+2)], lsurv[i], lse[i]))
+        y[i,] <- invtr(qnorm(ppoints(ny, a=0), lsurv[i], lse[i]))
     ## but actually calculate density at all y for every time, defining a grid on the whole plot region
     yy <- sort(unique(y))
     z <- matrix(nrow=n, ncol=length(yy))
@@ -159,7 +193,7 @@ densregion.normal <- function(x, mean, sd, ny=20, ...)
         stop(deparse(substitute(sd)), " and ", deparse(substitute(x)), " should be the same length")
     y <- matrix(nrow=n, ncol=ny)
     for (i in 1:n)
-        y[i,] <- qnorm(seq(0,1,length=ny+2)[-c(1,ny+2)], mean[i], sd[i])
+        y[i,] <- qnorm(ppoints(ny, a=0), mean[i], sd[i])
     yy <- sort(unique(y))
     z <- matrix(nrow=n, ncol=length(yy))
     for (i in 1:n)
@@ -179,13 +213,27 @@ seqToIntervals <- function(x){
 }
 
 sectioned.density <- function(x, dens, at, width, offset, ny,
-                              method=c("kernel","frequency"), nx, horiz=TRUE,
-                              colmax=par("fg"), colmin=par("bg"), gamma=1, ...)
+                              method=c("kernel","frequency"), nx, horiz=TRUE, up.left = TRUE,
+                              colmax, colmin, gamma=1, lattice=FALSE, ...)
 {
-    if (missing(width))
-        width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+    if (lattice) {
+        rect.fn <- panel.rect
+        default.width <- diff(trellis.last.object()[[if(horiz) "y.limits" else "x.limits"]]) / 20
+        default.colmax <- trellis.par.get("add.line")$col
+        default.colmin <- trellis.par.get("background")$col
+    }
+    else {
+        rect.fn <- rect
+        default.width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+        default.colmax <- par("fg")
+        default.colmin <- par("bg")
+    }
+    if (missing(width)) width <- default.width
+    if (missing(colmax)) colmax <- default.colmax
+    if (missing(colmin)) colmin <- default.colmin
     if (!is.numeric(x)) stop("\'x\' must be numeric")
     if (missing(offset)) offset <- width/3
+    if (!up.left) offset <- - offset
     if (!missing(dens)) {
         if (!is.numeric(dens)) stop("\'dens\' must be numeric")
         if (length(dens) != length(x)) stop("Lengths of \'dens\' and \'x\' must be the same")
@@ -217,49 +265,55 @@ sectioned.density <- function(x, dens, at, width, offset, ny,
         ints <- seqToIntervals(ind) # from R.utils
         for (j in seq(length=nrow(ints))) {
             if (horiz) 
-                rect(xleft=de$x[ints[j,1]],
+                rect.fn(xleft=de$x[ints[j,1]],
                      xright=de$x[ints[j,2]],
-                     ybottom=at + (i-1)*offset,
-                     ytop=at + (i-1)*offset + width,
-                     col=cols[i-1], border = (if(i==ny) cols[i-1] else NA) )
+                     ybottom=at + (i-1)*offset - (!up.left)*width,
+                     ytop=at + (i-1)*offset + up.left*width,
+                     col=cols[i-1], border = (if(i==ny) cols[i-2] else NA) )
             else
-                rect(ybottom=de$x[ints[j,1]],
+                rect.fn(ybottom=de$x[ints[j,1]],
                      ytop=de$x[ints[j,2]],
-                     xleft=at - (i-1)*offset,
-                     xright=at - (i-1)*offset - width,
-                     col=cols[i-1], border = (if(i==ny) cols[i-1] else NA) )
+                     xleft=at - (i-1)*offset - up.left*width,
+                     xright=at - (i-1)*offset + (!up.left)*width,
+                     col=cols[i-1], border = (if(i==ny) cols[i-2] else NA) )
         }
     }
     invisible()
 }
 
-cistrip <- function(x, at, d, horiz=TRUE, pch = 16, ...)
+cistrip <- function(x, at, d, horiz=TRUE, pch = 16, lattice=FALSE, ...)
 {
     if (missing(d)) d <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 60
     if (!is.numeric(x)) stop("\'x\' must be numeric")
     if (is.vector(x)) x <- matrix(x, ncol=3)
     n <- nrow(x)
     if (length(at) != n) stop("length of \'at\' should equal the number of estimates in \'x\'")
+    if (lattice) {
+        points.fn <- panel.points; seg.fn <- panel.segments;
+    }
+    else {
+        points.fn <- points; seg.fn <- segments;
+    }
     for (i in 1:n) { 
         if (horiz) { 
-            points(x[i,1], at[i], pch=pch, ...)
-            segments(x[i,2], at[i], x[i,3], at[i], ...)
-            segments(x[i,2], at[i]-d/2, x[i,2], at[i]+d/2, ...)
-            segments(x[i,3], at[i]-d/2, x[i,3], at[i]+d/2, ...)
+            points.fn(x[i,1], at[i], pch=pch, ...)
+            seg.fn(x[i,2], at[i], x[i,3], at[i], ...)
+            seg.fn(x[i,2], at[i]-d/2, x[i,2], at[i]+d/2, ...)
+            seg.fn(x[i,3], at[i]-d/2, x[i,3], at[i]+d/2, ...)
         }
         else {
-            points(at[i], x[i,1], pch=pch, ...)
-            segments(at[i], x[i,2], at[i], x[i,3], ...)
-            segments(at[i]-d/2, x[i,2], at[i]+d/2, x[i,2], ...)
-            segments(at[i]-d/2, x[i,3], at[i]+d/2, x[i,3], ...)
+            points.fn(at[i], x[i,1], pch=pch, ...)
+            seg.fn(at[i], x[i,2], at[i], x[i,3], ...)
+            seg.fn(at[i]-d/2, x[i,2], at[i]+d/2, x[i,2], ...)
+            seg.fn(at[i]-d/2, x[i,3], at[i]+d/2, x[i,3], ...)
         }
-        invisible()
     }
+    invisible()
 }
 
 vwstrip <- function(x, dens, at, width, horiz=TRUE, scale=1, limits=c(-Inf,Inf),
-                    col="gray", border=NULL, lwd=par("lwd"), lty=par("lty"),
-                    ticks=NULL, tlen=1, twd=par("lwd"), tty=par("lty"), ...) 
+                    col="gray", border=NULL, lwd, lty,
+                    ticks=NULL, tlen=1, twd, tty, lattice=FALSE, ...) 
 {
     if (!is.numeric(x)) stop("\'x\' must be numeric")
     if (missing(dens)) {
@@ -273,35 +327,53 @@ vwstrip <- function(x, dens, at, width, horiz=TRUE, scale=1, limits=c(-Inf,Inf),
         dens <- dens[order(x)]
         x <- sort(x)
     }
+    if (lattice) {
+        poly.fn <- panel.polygon; seg.fn <- panel.segments
+        default.width <- diff(trellis.last.object()[[if(horiz) "y.limits" else "x.limits"]]) / 20
+        default.lwd <- default.twd <- trellis.par.get("add.line")$lwd
+        default.lty <- default.tty <- trellis.par.get("add.line")$lty
+        default.border <- trellis.par.get("add.line")$col
+    }
+    else {
+        poly.fn <- polygon; seg.fn <- segments
+        default.width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+        default.lwd <- default.twd <- par("lwd")
+        default.lty <- default.tty <- par("lty")
+        default.border <- par("fg")
+    }
     dens <- dens[x>limits[1] & x<limits[2]]
     x <- x[x>limits[1] & x<limits[2]]
-    if (missing(width))
-        width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+    if (missing(width)) width <- default.width
+    if (missing(lwd)) lwd <- default.lwd
+    if (missing(lty)) lty <- default.lty
+    if (missing(twd)) twd <- default.twd
+    if (missing(tty)) tty <- default.tty
+    if (is.null(border)) border <- default.border
     dens <- dens / max(dens) * width/2 * scale
     xx <- c(x, rev(x))
     yy <- c(dens, -rev(dens)) + at 
-    if (horiz) polygon(xx, yy, col=col, border=border, lty=lty, lwd=lwd)
-    else polygon(yy, xx, col=col, border=border, lty=lty, lwd=lwd)
+    if (horiz) poly.fn(xx, yy, col=col, border=border, lty=lty, lwd=lwd)
+    else poly.fn(yy, xx, col=col, border=border, lty=lty, lwd=lwd)
     if (!is.null(ticks)) {
         dticks <- dens[findInterval(ticks, x)]
         if (horiz) { tx0 <- tx1 <- ticks; ty0 <- at - dticks*tlen; ty1 <- at + dticks*tlen }
         else { ty0 <- ty1 <- ticks; tx0 <- at - dticks*tlen; tx1 <- at + dticks*tlen }
-        segments(tx0, ty0, tx1, ty1, lwd=twd, lty=tty)
+        seg.fn(tx0, ty0, tx1, ty1, lwd=twd, lty=tty)
     }
     invisible()
 }
 
 vwstrip.normal <- function(mean, sd, log=FALSE, nx=1000, ...){    
     x <-
-        if (log) qlnorm(seq(0,1,length=nx)[-c(1,nx)], mean, sd)
-        else qnorm(seq(0,1,length=nx)[-c(1,nx)], mean, sd)
+        if (log) qlnorm(ppoints(nx, a=0), mean, sd)
+        else qnorm(ppoints(nx, a=0), mean, sd)
     dens <- if (log) dlnorm(x, mean, sd) else dnorm(x, mean, sd)
     vwstrip(x=x, dens=dens, ...)
 }
 
 bpstrip <- function(x, prob, at, width, horiz=TRUE, scale=1, limits=c(-Inf,Inf),
-                    col="gray", border=NULL, lwd=par("lwd"), lty=par("lty"),
-                    ticks=NULL, tlen=1, twd=par("lwd"), tty=par("lty"))
+                    col="gray", border=NULL, lwd, lty,
+                    ticks=NULL, tlen=1, twd, tty, lattice=FALSE)
 {
     if (!is.numeric(x)) stop("\'x\' must be numeric")
     if (missing(prob)) {
@@ -314,21 +386,80 @@ bpstrip <- function(x, prob, at, width, horiz=TRUE, scale=1, limits=c(-Inf,Inf),
         prob <- prob[order(x)]
         x <- sort(x)
     }
+    if (lattice) {
+        poly.fn <- panel.polygon; seg.fn <- panel.segments
+        default.width <- diff(trellis.last.object()[[if(horiz) "y.limits" else "x.limits"]]) / 20
+        default.lwd <- default.twd <- trellis.par.get("add.line")$lwd
+        default.lty <- default.tty <- trellis.par.get("add.line")$lty
+        default.border <- trellis.par.get("add.line")$col
+    }
+    else {
+        poly.fn <- polygon; seg.fn <- segments
+        default.width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+        default.lwd <- default.twd <- par("lwd")
+        default.lty <- default.tty <- par("lty")
+        default.border <- par("fg")
+    }
     prob <- pmin(prob, 1 - prob)
     prob <- prob[x>limits[1] & x<limits[2]]
     x <- x[x>limits[1] & x<limits[2]]
-    if (missing(width))
-        width <- diff(par("usr")[if(horiz) 3:4 else 1:2]) / 20
+    if (missing(width)) width <- default.width
+    if (missing(lwd)) lwd <- default.lwd
+    if (missing(lty)) lty <- default.lty
+    if (missing(twd)) twd <- default.twd
+    if (missing(tty)) tty <- default.tty
+    if (is.null(border)) border <- default.border
     prob <- prob / max(prob) * width/2 * scale
     xx <- c(x, rev(x))
     yy <- c(prob, -rev(prob)) + at 
-    if (horiz) polygon(xx, yy, col=col, border=border, lty=lty, lwd=lwd)
-    else polygon(yy, xx, col=col, border=border, lty=lty, lwd=lwd)
+    if (horiz) poly.fn(xx, yy, col=col, border=border, lty=lty, lwd=lwd)
+    else poly.fn(yy, xx, col=col, border=border, lty=lty, lwd=lwd)
     if (!is.null(ticks)) {
         pticks <- prob[findInterval(ticks, x)]
         if (horiz) { tx0 <- tx1 <- ticks; ty0 <- at - pticks*tlen; ty1 <- at + pticks*tlen }
         else { ty0 <- ty1 <- ticks; tx0 <- at - pticks*tlen; tx1 <- at + pticks*tlen }
-        segments(tx0, ty0, tx1, ty1, lwd=twd, lty=tty)
+        seg.fn(tx0, ty0, tx1, ty1, lwd=twd, lty=tty)
     }
     invisible()
 }
+
+panel.denstrip <- function(...)
+{
+    denstrip(..., lattice=TRUE)
+}
+
+panel.denstrip.normal <- function(...)
+{
+    denstrip.normal(..., lattice=TRUE)
+}
+
+panel.denstrip.legend <- function(...)
+{
+    denstrip.legend(..., lattice=TRUE)
+}
+
+panel.cistrip <- function(...)
+{
+    cistrip(..., lattice=TRUE)
+}
+
+panel.vwstrip <- function(...)
+{
+    vwstrip(..., lattice=TRUE)
+}
+
+panel.vwstrip.normal <- function(...)
+{
+    vwstrip.normal(..., lattice=TRUE)
+}
+
+panel.bpstrip <- function(...)
+{
+    bpstrip(..., lattice=TRUE)
+}
+
+panel.sectioned.density <- function(...)
+{
+    sectioned.density(..., lattice=TRUE)
+}
+
